// SNF4CGP/ConfigurationEngine.hpp
// Copyright (C) 2009 ARM Research Labs, LLC
// See www.armresearch.com for more information.
//
// This module parses the configuration and keeps track of it's generation.
// When asked it can produce the appropriate configuration for a given Job
// based on the scan result that was recieved. If it is asked and the current
// configuration is out of date then the new configuration is parsed first.

#ifndef IncludedConfigurationEngine
#define IncludedConfigurationEngine

#include <string>
#include <vector>

using namespace std;

class snf_RulebaseHandler;

class ResultConfiguration {
  public:
    enum ActionType {
        Bypass,
        Allow,
        Reject,
        Delete,
        Hold
    };

    int ResultCode;
    ActionType Action;
    bool InjectHeaders;
    bool EmitXMLLog;
    bool EmitClassicLog;
    string LogComment;
    string RejectionReason;
    string HoldPath;

    ResultConfiguration() :
      ResultCode(0),
      Action(Bypass),
      InjectHeaders(false),
      EmitXMLLog(false),
      EmitClassicLog(false),
      LogComment(""),
      RejectionReason(""),
      HoldPath("") {
    }

    ResultConfiguration(const ResultConfiguration& R) :
      ResultCode(R.ResultCode),
      Action(R.Action),
      InjectHeaders(R.InjectHeaders),
      EmitXMLLog(R.EmitXMLLog),
      EmitClassicLog(R.EmitClassicLog),
      LogComment(R.LogComment),
      RejectionReason(R.RejectionReason),
      HoldPath(R.HoldPath) {
    }

    ResultConfiguration& operator=(const ResultConfiguration& R) {
      ResultCode = R.ResultCode;
      Action = R.Action;
      InjectHeaders = R.InjectHeaders;
      EmitXMLLog = R.EmitXMLLog;
      EmitClassicLog = R.EmitClassicLog;
      LogComment = R.LogComment;
      RejectionReason = R.RejectionReason;
      HoldPath = R.HoldPath;
      return (*this);
    }
};

const int HighestResultCode = 63;
const int ResultConfigurationSize = HighestResultCode + 1;

class ConfigurationManager {
  private:
    snf_RulebaseHandler& Rulebase;
    int Generation_;
    vector<ResultConfiguration> Configurations;

  public:
    ConfigurationManager(snf_RulebaseHandler& R);
    bool isOutOfDate();
    void update();
    ResultConfiguration& ConfigurationForResultCode(int Code);
};

#endif
